/*----------------------------------------------------------------------------
 * image-psnr - Measure PSNR of two pictures
 * Copyright (C) 2008 Arne Bochem <image-psnr@ps-auxw.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "read_image.h"

double get_psnr (unsigned char *a, unsigned char *b, int count, int offset, int step_size)
{
	double mse = 0;
	double max = pow(2, 8) - 1;
	int i;

	for (i = 0; i < count; i += step_size)
	{
		mse += pow(a[i + offset] - b[i + offset], 2);
	}

	return 20 * log(max / sqrt(mse / (double)(count / step_size))) / log(10);
}

int main (int argc, char *argv[])
{
	char *channel_names[3] = {"Red:   ", "Green: ", "Blue:  "};
	frames_t *first, *second;
	int w, h, w2, h2, length;
	int channel;
	double psnr;

	if (argc != 3)
	{
		fprintf(stderr, "Usage: image-psnr IMAGE IMAGE\n\nBoth images have to have the same dimensions.\nSupported formats: BMP, GIF, JPEG, PNG, TGA\nAlpha channels are ignored. 8bit per channel only.\nAnimations must have the same number of frames.\n");
		return 0;
	}

	if ((first = read_image(argv[1], &w, &h)) == NULL)
		abort();

	if ((second = read_image(argv[2], &w2, &h2)) == NULL)
		abort();

	if (w != w2 || h != h2)
	{
		fprintf(stderr, "Error: Images have different dimensions: %dx%d - %dx%d\n", w, h, w2, h2);
		abort();
	}

	if (first->number != second->number)
	{
		fprintf(stderr, "Error: Different number of frames in animations: %d - %d\n", first->number, second->number);
		abort();
	}

	length = w * h * first->number * 3;

	for (channel = 0; channel < 3; channel++)
	{
		psnr = get_psnr(first->frames[0], second->frames[0], length, channel, 3);
		printf("%s %fdb\n", channel_names[channel], psnr);
	}

	psnr = get_psnr(first->frames[0], second->frames[0], length, 0, 1);
	printf("Global: %fdb\n", psnr);

	return 0;
}
