/*----------------------------------------------------------------------------
 * libimgloader - Loading images made simple
 * Copyright (C) 2008 Arne Bochem <libimgloader@arneb.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#ifndef __BMP_COMMON_H
#define __BMP_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

#include "imgloader_common.h"

#define BMP_COMP_RGB       0
#define BMP_COMP_RLE8      1
#define BMP_COMP_RLE4      2
#define BMP_COMP_BITFIELDS 3
#define BMP_COMP_JPEG      4
#define BMP_COMP_PNG       5

#define BMP_1BPP_MASK      0x01
#define BMP_4BPP_MASK      0x0f
#define BMP_8BPP_MASK      0xff

/* Fun field, used to get mask for bit depth */

static uint8_t bmp_bpp_masks[9] = {0, BMP_1BPP_MASK, 0, 0, BMP_4BPP_MASK, 0, 0, 0, BMP_8BPP_MASK};

/* Global file header */

struct bmp_file_header
{
	uint16_t type;
	uint32_t size;
	uint16_t reserved_1; /* must be 0 */
	uint16_t reserved_2; /* must be 0 */
	uint32_t bmp_offset;
} __attribute__ ((packed));

/* These headers don't include the header size (uint32_t) at the beinning. Read it to determine the header type. */

struct bmp_info_header_win_v3 /* len == "40" */
{
	int32_t width;
	int32_t height;
	uint16_t planes;           /* must be 1 */
	uint16_t depth;            /* 1 4 8 16 24 32 */
	uint32_t compression;      /* one of the BMP_COMP_* defines */
	uint32_t image_size;       /* raw bitmap data size */
	uint32_t ppm_horiz;        /* pixels per meter, horizontal */
	uint32_t ppm_vert;         /* pixels per meter, horizontal */
	uint32_t colors_used;      /* number of colors used. if 0, number of colors is 2^depth */
	uint32_t colors_important; /* number of 'important' colors. if 0, all colors are 'important' */
} __attribute__ ((packed));

struct bmp_info_header_os2_v1 /* len == "12" */
{
	uint16_t width;
	uint16_t height;
	uint16_t planes; /* must be 1 */
	uint16_t depth;  /* 1 4 8 24 */
} __attribute__ ((packed));

/* Multiple header styles? WTF. Anyway. If you need support for other formats, which this implementation doesn't support yet, tell me. Also, send a sample. Motivation too. */

#ifdef __cplusplus
}
#endif

#endif
