/*----------------------------------------------------------------------------
 * libimgloader - Loading images made simple
 * Copyright (C) 2008 Arne Bochem <libimgloader@arneb.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#include "imgloader_common.h"

/* result = alloc_images(number of frames, width, height) */
frames_t *alloc_images(int n, int w, int h)
{
	frames_t *result;
	int i;

	/* Allocate and set up image structure result */
	FATALLOC(result, sizeof(frames_t) + n * (sizeof(uint8_t *) + sizeof(int) + w * h * 3), "image"); /* structure + number * (*frame + delay + image) */

	/* Set number of frames */
	result->number = n;

	/* Set up pointer to delays */
	result->delays = (int *)((uint8_t *)result + sizeof(frames_t));
	result->frames = (uint8_t **)((uint8_t *)(result->delays) + n * sizeof(int));

	for (i = 0; i < n; i++)
	{
		result->delays[i] = 100; /* 100ms should be a sensible default */
		result->frames[i] = (uint8_t *)(result->frames) + n * sizeof(uint8_t *) + i * w * h * 3;
	}

	return result;
}
