/*----------------------------------------------------------------------------
 * libimgloader - Loading images made simple
 * Copyright (C) 2008 Arne Bochem <libimgloader@arneb.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#ifndef __IMGLOADER_H
#define __IMGLOADER_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <limits.h>

/* Macros */

/* Wicked code */
#define FATALLOC(target, memory, name) {if (((int)memory < 2) || ((unsigned int)memory > INT_MAX - 1024)) \
	{ \
		fprintf(stderr, "Error: Possible integer overflow in " name " malloc detected: %u\n", memory); \
		abort(); \
	} \
	if ((target = malloc(memory)) == NULL) \
	{ \
		perror("Cannot allocate memory for " name); \
		abort(); \
	}}

/* Types */

typedef struct frames_s
{
	uint8_t **frames;
	int *delays;
	int number; /* of frames */
} frames_t;

typedef frames_t *(*image_reader)(char *filename, int *w, int *h);

/* Utility functions */

frames_t *alloc_images(int number, int w, int h);

#ifdef __cplusplus
}
#endif

#endif
