/*----------------------------------------------------------------------------
 * libimgloader - Loading images made simple
 * Copyright (C) 2008 Arne Bochem <libimgloader@arneb.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#include "imgloader_common.h"
#include "read_image.h"
#include "read_targa.h"
#include "read_jpeg.h"
#include "read_png.h"
#include "read_gif.h"
#include "read_bmp.h"

/* Identify file type by magic bytes. Takes a filename and returns function pointer to loader. */
image_reader identify_by_magic(char *filename)
{
	uint8_t buffer[13] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
	FILE *fh;

	if ((fh = fopen(filename, "r")) == NULL)
		return NULL;

	fread(buffer, 1, 12, fh);
	fclose(fh);

	/* PNG */
	if (!strncmp(buffer, "\x89PNG", 4))
		return read_png;

	/* JPEG */
	if (!strncmp(buffer, "\377\330\377", 3))
		return read_jpeg;

	/* JPEG2000 */
	/*if (!strncmp(buffer, "\x00\x00\x00\x0C\x6A\x50\x20\x20\x0D\x0A\x87\x0A", 12))
		return read_jpeg2000;*/

	/* GIF */
	if (!strncmp(buffer, "GIF8", 4))
		return read_gif;

	/* BMP */
	if (!strncmp(buffer, "BM", 2))
		return read_bmp;

	/* Targa */
	if (((buffer[2] == 2 || buffer[2] == 10) && buffer[1] == 0) || ((buffer[2] == 1 || buffer[2] == 9) && buffer[1] == 1))
		return read_targa;

	return NULL;
}

frames_t *read_image(char *filename, int *w, int *h)
{
	image_reader reader = identify_by_magic(filename);
	frames_t *result;

	if (reader == NULL)
	{
		fprintf(stderr, "The file %s does not seem to be of a supported image type.\n", filename);
		return NULL;
	}

	result = reader(filename, w, h);

	if (result == NULL)
		fprintf(stderr, "The file %s could not be decoded.\n", filename);

	return result;
}
