/*----------------------------------------------------------------------------
 * libimgloader - Loading images made simple
 * Copyright (C) 2008 Arne Bochem <libimgloader@arneb.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#include "imgloader_common.h"
#include <jpeglib.h>
#include "read_jpeg.h"

/* Code */

frames_t *read_jpeg(char *filename, int *w, int *h)
{
	frames_t *result;
	FILE *fh;
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
	JSAMPROW row_pointer;
	JSAMPARRAY scanlines = &row_pointer;
	uint8_t *image;

	/* Initialize error handler and decompressor */
	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

	/* Open file */
	if ((fh = fopen(filename, "rb")) == NULL)
	{
		perror("Cannot open JPEG file for reading");
		return NULL;
	}

	/* Set data source */
	jpeg_stdio_src(&cinfo, fh);

	/* Read file header */
	jpeg_read_header(&cinfo, TRUE);

	/* Gather information */
	*w = cinfo.image_width;
	*h = cinfo.image_height;

	/* Allocate image memory */
	result = alloc_images(1, *w, *h);
	image = result->frames[0];

	/* Initialize row pointer */
	row_pointer = image;

	/* Set decompression parameters */
	cinfo.out_color_space = JCS_RGB;

	/* Start decompression */
	jpeg_start_decompress(&cinfo);

	/* Decompress scanlines */
	while (cinfo.output_scanline < cinfo.output_height)
	{
		jpeg_read_scanlines(&cinfo, scanlines, 1);
		*scanlines += *w * 3;
	}

	/* Finish decompression */
	jpeg_finish_decompress(&cinfo);

	/* Close filehandle */
	fclose(fh);

	/* Free decompressor */
	jpeg_destroy_decompress(&cinfo);

	return(result);
}
