/*----------------------------------------------------------------------------
 * libimgloader - Loading images made simple
 * Copyright (C) 2008 Arne Bochem <libimgloader@arneb.de>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *----------------------------------------------------------------------------*/

#ifndef __TARGA_COMMON_H
#define __TARGA_COMMON_H

#ifdef __cplusplus
extern "C" {
#endif

/* Bitmasks */
#define TARGA_16_RED(x) (((x) & 0xf8) >> 2)
#define TARGA_16_GREEN(x) ((((x) & 0x3) << 3)|((x) & 0xe000)>>13)
#define TARGA_16_BLUE(x) (((x) & 0x1f00) >> 8)
#define TARGA_16_ALPHA(x) (((x) & 0x80) >> 7)

#define TARGA_IS_RLE(x) ((x) & 0x80)
#define TARGA_PACK_LEN(x) (((x) & 0x7f) + 1)

#define TARGA_COL_ORIGIN(x) ((x) & 16)
#define TARGA_LINE_ORIGIN(x) ((x) & 32)

/* Structures */
struct targa_file
{
	uint8_t id_field_len;
	uint8_t color_map_type; /* == 0 && frames_type == 2 or 10 => skip color_map */
	uint8_t frames_type; /* support: 1, 2, 9, 10 */
	uint16_t color_map_origin;
	uint16_t color_map_length;
	uint8_t color_map_entry_size;
	uint16_t x_origin;
	uint16_t y_origin;
	uint16_t width;
	uint16_t height;
	uint8_t pixel_size;
	uint8_t image_descriptor;
	uint8_t *image_id; /* id_field_len */
	uint8_t *color_map; /* length: color_map_entry_size/8 * color_map_length */
	uint8_t *image_data; /* length: get rest */
} __attribute__ ((packed));

#ifdef __cplusplus
}
#endif

#endif
